@@ -1,5 +1,5 @@
+from .base import ConcatLibrary
 from .custom_library import CustomLibrary
-from .feature_library import ConcatLibrary
 from .fourier_library import FourierLibrary
 from .identity_library import IdentityLibrary
 from .polynomial_library import PolynomialLibrary
