@@ -3,5 +3,13 @@ from .constrained_sr3 import ConstrainedSR3
 from .sindy_optimizer import SINDyOptimizer
 from .sr3 import SR3
 from .stlsq import STLSQ
+from .trapping_sr3 import TrappingSR3
 
-__all__ = ["BaseOptimizer", "ConstrainedSR3", "SINDyOptimizer", "SR3", "STLSQ"]
+__all__ = [
+    "BaseOptimizer",
+    "SINDyOptimizer",
+    "SR3",
+    "STLSQ",
+    "ConstrainedSR3",
+    "TrappingSR3",
+]
